/* collisionset.c++ Stores collision set, that is one half of 
                    the object that are involved into collsion
                    detection.
                    Physical Based Simulation, ETH Zuerich
                    Rene Mueller, June 16th, 2003 */

#include <iostream>
#include "collisionset.h"

//! empty constructor
CollisionSet::CollisionSet()
{
   root = new ModelList;
   root->next = NULL;
   root->model = NULL;

   // set orientation to identity matrix
   orientation[0][0] = 1.0; orientation[0][1] = 0.0; orientation[0][2] = 0.0;
   orientation[1][0] = 0.0; orientation[1][1] = 1.0; orientation[1][2] = 0.0;
   orientation[2][0] = 0.0; orientation[2][1] = 0.0; orientation[2][2] = 1.0;
   
   // set translation to zero vector
   translation[0] = 0.0; 
   translation[1] = 0.0; 
   translation[2] = 0.0; 
}



//! insert object into collision set
bool CollisionSet::insert(Model* model)
{
   if (contains(model)) {
      // model is already in set 
      return false;
   }

   // find end of list
   ModelList* ptr = root;
   while (ptr->next != NULL) {
      ptr = ptr->next;
   }
   // ptr.next == NULL now 
   ptr->next = new ModelList;
   ptr->next->next = NULL;
   ptr->next->model = model;
   
   return true;
}


//! check wether collision set contains given object
bool CollisionSet::contains(Model* model)
{
   ModelList* ptr = root;
   while ((ptr != NULL) && (ptr->model != model)) {
      ptr = ptr->next;
   }
   if (ptr == NULL) {
      return false;
   } else {
      return (ptr->model == model);
   }
}


//! set translation matrix 
void CollisionSet::setTranslation(double t[3])
{
   for (int i=0; i<3; i++) {
      translation[i] = t[i];
   }
}   


//! set orientation matrix
void CollisionSet::setOrientation(double o[3][3])
{
   for (int i=0; i<3; i++) {
      for (int j=0; j<3; j++) {
         orientation[i][j] = o[i][j];
      }
   }
}   


//! destructor
CollisionSet::~CollisionSet() { }


//! check for collision
bool CollisionSet::checkForCollision(CollisionSet *setA, CollisionSet *setB)
{
   bool collision = false;
   ModelList* modelA, *modelB;
   for (modelA = setA->root->next; modelA != NULL; modelA = modelA->next) {
      for (modelB = setB->root->next; modelB != NULL; modelB = modelB->next) {
         Model* A = modelA->model;
         Model* B = modelB->model;
         
         RAPID_Collide(setA->orientation, setA->translation, 
            A->rapidModel, setB->orientation, setB->translation, B->rapidModel,
            RAPID_FIRST_CONTACT);
         collision = (RAPID_num_contacts != 0);
         if (collision) {
             return true;
         }
      }
   }

   return collision;
}
