/* collisionset.h  stores collision set, that is one half of 
                   the object that are involved into collsion
                   detection.
                   Physical Based Simulation, ETH Zuerich
                   Rene Mueller, June 16th, 2003 */

#ifndef _RAPID_H_
#define _RAPID_H_

#include <RAPID.H>
#include "model.h"

//! this data structure defines a linked list of Model 
typedef struct ModelList {
   //! next reference in list 
   ModelList* next;

   //! reference to actual model (payload reference)
   Model* model;    
} ModelList;

//! class represents set of collisionable objects
/*!
 *  \class CollisionSet
 *  This class represents a data structure to store 
 *  Model instances that are involved in the collision
 *  detection process.
 * 
 *  \author Rene Mueller
 *  \date June 16th, 2003 
 */
class CollisionSet {
public:

//! empty constructor
CollisionSet();

//! insert object into collision set
/*!
 *  insert Model into collision set if it does not already exist
 *  \param model Model to be inserted
 *  \return true  if model was successfully inserted and 
 *          false if model is already in set
 */
bool insert(Model* model);

//! check wether collision set contains given object
/*!
 *  check whether given model is present in set 
 *  \param model model to check
 *  \return true if model is in set, false otherwise
 */
bool contains(Model* model);

//! set translation matrix 
/*!
 *  \param t new translation matrix (copied by value)
 */
void setTranslation(double t[3]);


//! set orientation matrix
/*!
 * \param o new orientation matrix (copied by value)
 */
void setOrientation(double o[3][3]); 

//! check for collision
/*!
 *  check every model from each set against every model from the other 
 *  set if there is a collision return true, otherwise false.
 *  \param setA first collision set 
 *  \param setB second collision set
 *  \return true if there is a collision, false otherwise 
 */
static bool checkForCollision(CollisionSet *setA, CollisionSet *setB);


//! translation matrix
double translation[3];

//! orientation matrix
double orientation[3][3];

//! destructor
~CollisionSet();


private:

   // root node of linked list 
   ModelList* root;

   
};
   
#endif // _RAPID_H_

