/* model.h  represents data model 
            (Inventor model and RAPID model)
            Physical Based Simulation, ETH Zuerich
            Rene Mueller, April 14th 2003 */

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <RAPID.H>

#ifndef _MODEL_H_
#define _MODEL_H_

//! model represents Inventor and RAPID model
/*!
 * \class Model
 * 
 *  This class represents the OpenInventor and the 
 *  RAPID model used for collision detection.
 *
 *  \author Rene Mueller, April 14th 2003
 */
class Model 
{
public:

   //! read new model from file 
   /*!
    *  read 3D data from IV file and create 
    *  Inventor and RAPID model
    *  \param filename file name to read in
    */
   Model(const char* filename);

   //! destructor
   ~Model();

   //! return Inventor model with 3D data
   SoSeparator* getModel() {
      return model;
   }

   //! return Inventor root model (children: model, node, material node)
   SoSeparator* getRootModel() {
      return rootModel;
   }

   //! return Inventor material node 
   SoMaterial* getMaterial() {
      return material;
   }

   //! return RAPID model
   RAPID_model* getRAPIDModel() {
      return rapidModel;
   }
     
   //! RAPID collision detection model
   RAPID_model* rapidModel;

private:
   
   //! Open Inventor model 
   SoSeparator* model;

   //! Open Inventor model root (children: model node, material node)
   SoSeparator* rootModel;

   //! Open Inventor material node
   SoMaterial* material;
   
};

#endif // _MODEL_H_
