/* romanglepool.c++ Stores the ROM values obtained by the simulation 
                    Physical Based Simulation, ETH Zuerich
                    Rene Mueller, June 16th, 2003 */

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include "romanglepool.h"

#define CONE_LENGTH 100

void RomAnglePool::append(float phi, float theta) {
   PoolElement* poolElement = new PoolElement();
   poolElement->phi       = phi;
   poolElement->theta     = theta;
   poolElementVector.insert(poolElementVector.end(), poolElement);
}

int RomAnglePool::length() {
   return poolElementVector.size();
}

void RomAnglePool::get(int index, float *phi, float *theta) {
   PoolElement *poolElement;
   if (index < poolElementVector.size()) {
      poolElement = poolElementVector[index];
      *phi       = poolElement->phi;
      *theta     = poolElement->theta;
   }
}

SoNode* RomAnglePool::createVisualizationCone() const 
{
   SoSeparator *cone = new SoSeparator;
   SoCoordinate3 *coords = new SoCoordinate3;
   SoIndexedFaceSet *faces = new SoIndexedFaceSet;

   // add tip of cone
   coords->point.set1Value(0, 0, 0, 0);
   for (int i=0; i<poolElementVector.size(); i++) {
      double phi = poolElementVector[i]->phi*M_PI/180.0;
      double theta = poolElementVector[i]->theta*M_PI/180.0;
      double x = -sin(theta)*cos(phi)*CONE_LENGTH;
      double y = -cos(theta)*CONE_LENGTH; 
      double z = -sin(theta)*sin(phi)*CONE_LENGTH;
      coords->point.set1Value(i+1, x, y, z);
   }

   // build up cone faces
   for (int i=0; i<poolElementVector.size(); i++) {
      faces->coordIndex.set1Value(4*i, i+1); // first point 
      faces->coordIndex.set1Value(4*i+1, (i+1)%poolElementVector.size()+1); // second point
      faces->coordIndex.set1Value(4*i+2, 0); // cone tip
      faces->coordIndex.set1Value(4*i+3, -1); // end of face
   }
   
   cone->addChild(coords);
   cone->addChild(faces);
   
   return cone;
}
