/* romanglepool.h Stores the ROM values obtained by the simulation 
                  Physical Based Simulation, ETH Zuerich
                  Rene Mueller, June 16th, 2003 */
                                                                                
#ifndef _ROMANGLEPOOL_H_
#define _ROMANGLEPOOL_H_

#include <Inventor/nodes/SoNode.h>
#include <iostream>
#include <vector>

//! This class saves the angles of the Rom.

class RomAnglePool {
public:
   //! Constructor of this class
   RomAnglePool() {};

   //! Destructor of this class   
   ~RomAnglePool() {};

   //! puts the parameters into a struct and append it to the internal list
   /*!
       \param phi specifies the angle phi
       \param theta specifies the angle theta
   */  
   void append(float phi, float theta);

   //! return the length of the internal vector
   int length();

   //! returns the values of the internal stuct at the position index
   /*!
       \param index list index whose values should be returned
       \param phi specifies the angle phi
       \param theta specifies the angle theta
   */
   void get(int index, float *phi, float *theta);

   //! create ROM visualization cone 
   /*!
       Use the data points that are already in the pool and create
       the SGI OpenInventor visualization cone of the Range Of Motion. 
       \return SoNode of ROM cone
    */
   SoNode* createVisualizationCone() const;  
   

private:   

   //! The struct PoolElement defines one entry in the pool
   typedef struct {
      float phi;
      float theta;
   } PoolElement;

   //! poolElemenetVector is a vector of PoolElement pointers 
   std::vector<PoolElement*> poolElementVector;
};

#endif // _ROMANGLEPOOL_H_

